﻿using System.Collections;
using System;
using UnityEngine;

namespace Mod
{
    public class Mod : MonoBehaviour
    {
        public static void Main()
        {
            const string modTag = " [MLM]";
            const string mlID = "Molten Lava" + modTag;
            ModAPI.RegisterLiquid(mlID, new MoltenLava());

            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Blood Flask"),
                    NameOverride = "Molten Lava Flask" + modTag,
                    DescriptionOverride = "Is hot. Very hot.",
                    CategoryOverride = ModAPI.FindCategory("Biohazard"),
                    ThumbnailOverride = ModAPI.LoadSprite("sprites/LavaFlaskThumbnail.png"),
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<FlaskBehaviour>().StartLiquid = new BloodContainer.SerialisableDistribution
                        {
                            LiquidID = mlID,
                            Amount = 2.8f //a flask can only hold 2.8 units of liquid (1 L)
                        };
                    }
                }
            );
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Acid Syringe"),
                    NameOverride = "Molten Lava Syringe" + modTag,
                    DescriptionOverride = "Honestly this seems unethical.",
                    CategoryOverride = ModAPI.FindCategory("Biohazard"),
                    ThumbnailOverride = ModAPI.LoadSprite("sprites/LavaSyringeThumbnail.png"),
                    AfterSpawn = (Instance) =>
                    {
                        UnityEngine.Object.Destroy(Instance.GetComponent<SyringeBehaviour>());
                        Instance.GetOrAddComponent<MoltenLavaSyringe>();
                    }
                }
            );
        }
    }

    public class MoltenLavaSyringe : SyringeBehaviour
    {
        // provide the liquid ID for this syringe
        public override string GetLiquidID() => "Molten Lava [MLM]";
    }

    public class MoltenLava : TemporaryBodyLiquid
    {
        public const string ID = "Molten Lava [MLM]";

        public float RemovalChancePerSecond = 0.1f;
        public virtual bool ShouldCallOnEnterEveryUpdate => true;
        private Color color = new UnityEngine.Color(1f, 0.4f, 0.2f);

        public MoltenLava()
        {
            Color = new UnityEngine.Color(1f, 0.4f, 0.2f);
        }

        public override void OnUpdate(BloodContainer c)
        {
            base.OnUpdate(c);
            if (Global.main.PhysicalObjectsInWorldByTransform.TryGetValue(c.transform, out var phys))
            {
                if (phys.SimulateTemperature)
                    phys.Temperature = 1000;
            }
        }

        public override void OnEnterLimb(LimbBehaviour limb)
        {
            limb.PhysicalBehaviour.Temperature = 1000;
        }
        //var light = ModAPI.CreateLight(container.transform, Color.red, 5, 2);
        public override void OnEnterContainer(BloodContainer container)
        {
            container.gameObject.GetOrAddComponent<ContainsLavaBehavior>();
        }

        public override void OnExitContainer(BloodContainer container)
        {
            ContainsLavaBehavior component;
            if (!container.TryGetComponent<ContainsLavaBehavior>(out component))
                return;
            UnityEngine.Object.Destroy((UnityEngine.Object)component);
        }
    }

    public class ContainsLavaBehavior : MonoBehaviour
    {
        private GameObject glow;
        private LightSprite lightSprite;

        private void Start()
        {
            this.glow = new GameObject("ow my bones", new Type[1]
            {
                typeof (Optout)
            });
            this.glow.transform.SetParent(this.transform);
            this.glow.transform.localPosition = new Vector3();
            this.glow.transform.localRotation = new Quaternion();
            this.glow.transform.localScale = Vector3.one;
            float radius = 5f;
            float brightness = 2f;
            Color color = new UnityEngine.Color(1f, 0.4f, 0.2f);
            PhysicalBehaviour component;
            if (this.TryGetComponent<PhysicalBehaviour>(out component))
                radius = Mathf.Sqrt(component.ObjectArea) * 3f;
            this.lightSprite = ModAPI.CreateLight(this.glow.transform, color, radius, brightness);
        }

        private void OnDestroy()
        {
            if ((bool)(UnityEngine.Object)this.lightSprite)
                UnityEngine.Object.Destroy((UnityEngine.Object)this.lightSprite);
            if (!(bool)(UnityEngine.Object)this.glow)
                return;
            UnityEngine.Object.Destroy((UnityEngine.Object)this.glow);
        }
    }
}